"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Computed = void 0;
const internal_1 = require("../internal");
class Computed extends internal_1.State {
    constructor(agileInstance, computeFunction, config = {}) {
        super(agileInstance, computeFunction(), {
            key: config.key,
            dependents: config.dependents,
        });
        this.deps = [];
        this.hardCodedDeps = [];
        config = internal_1.defineConfig(config, {
            computedDeps: [],
        });
        this.agileInstance = () => agileInstance;
        this.computeFunction = computeFunction;
        this.hardCodedDeps = this.formatDeps(config.computedDeps);
        this.deps = this.hardCodedDeps;
        this.recompute();
    }
    recompute(config = {}) {
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
        });
        this.ingest(config);
    }
    updateComputeFunction(computeFunction, deps = [], config = {}) {
        config = internal_1.defineConfig(config, {
            background: false,
            sideEffects: {
                enabled: true,
                exclude: [],
            },
            overwriteDeps: true,
        });
        const newDeps = this.formatDeps(deps);
        if (config.overwriteDeps)
            this.hardCodedDeps = newDeps;
        else
            this.hardCodedDeps = this.hardCodedDeps.concat(newDeps);
        this.deps = this.hardCodedDeps;
        this.computeFunction = computeFunction;
        this.recompute({
            background: config.background,
            sideEffects: config.sideEffects,
        });
    }
    compute() {
        internal_1.ComputedTracker.track();
        const computedValue = this.computeFunction();
        const foundDeps = internal_1.ComputedTracker.getTrackedObservers();
        const newDeps = [];
        this.hardCodedDeps.concat(foundDeps).forEach((observer) => {
            newDeps.push(observer);
            observer.depend(this.observer);
        });
        this.deps = newDeps;
        return computedValue;
    }
    formatDeps(instances) {
        const finalInstances = [];
        for (const instance of instances) {
            if (instance instanceof internal_1.Observer) {
                finalInstances.push(instance);
                continue;
            }
            if (instance !== undefined &&
                instance['observer'] !== undefined &&
                instance['observer'] instanceof internal_1.Observer)
                finalInstances.push(instance['observer']);
        }
        return finalInstances;
    }
    patch() {
        internal_1.Agile.logger.error("You can't use patch method on ComputedState!");
        return this;
    }
    persist() {
        internal_1.Agile.logger.error("You can't use persist method on ComputedState!");
        return this;
    }
    invert() {
        internal_1.Agile.logger.error("You can't use invert method on ComputedState!");
        return this;
    }
}
exports.Computed = Computed;
